<?php
#
# Script         The Learning Script
# Copyright      2013 Denny Schlesinger
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# Autor          Denny Schlesinger
#                http://softwaretimes.com
#
# $Id: $
#

register_shutdown_function('shutdown'); 

function shutdown() { 
     $a = error_get_last(); 
     if($a !== null) {
         echo "<br>I can't learn too many new prime numbers all at once,<br>\n";
         echo "it gives my computer brain a headache and it shuts down.<br>\n";
         echo "Try a smaller number.\n";
     }
 } 


function primeSieve($limit, &$prime, $url) {

    $max = max($prime);       // highest prime number in the array
    $i   = $max + 2;          // start checking next higher odd number
    while($limit > $max) {    // loop from current high prime number to limit
        $test = $i;           // initialize variables in loop
        $isPrime = TRUE;      // initialize variables in loop
        foreach($prime AS $value) {
            if ($test % $value == 0) { // if there is an even divisor
                $isPrime = FALSE;      // not a prime number
                break;                 // exit foreach loop
            }
        }
        if($isPrime) {        // if it is a prime number
            $prime[] = $i;    // add it to the array
            $max = $i;        // new highest prime number
        }
        if ($i+2 % 5 == 0) {  // if next odd number ends in 5 or 0
            $i += 4;          // increment by 4
        } else {              // else
            $i += 2;          // increment by 2 (don't check even numbers)
        }
    } // while($limit > $max)
    
    // save prime numbers to file
    file_put_contents ($url, serialize($prime));

} // function primeSieve()

function checkTime($action, &$timer, $limit = 0) {

    switch($action) {
    
    case 'PrimeS':
        $timer['PrimeS'] = $limit;
        break;
        
    case 'PrimeE':
        $timer['PrimeE'] = $limit;
        break;
        
    case 'PrimeNew':
        return $timer['PrimeE'] - $timer['PrimeS'];
        
    case 'Start':
        $timer['start'] = microtime(true);
        break;
        
    case 'End':
        $timer['end'] = microtime(true);
        break;
        
    case 'Time':
        return $timer['end'] - $timer['start'];
        
    case 'Limit':
        if(($timer['end'] - $timer['start']) > $limit) {
            return TRUE;
        } else {
            return FALSE;
        }

    } // switch($action)

} // function checkTime()


// initialize variable
$url = 'prime-numbers.php';
$feedback = '';
$input = '';
$timer = array();
$timeLimit = 5;  // seconds

switch ($_SERVER['REQUEST_METHOD']) {

case 'POST':

    // sanitize input
    $input = floor(preg_replace( '/[^0-9\.]/', '', $_POST['number']));
    
    // get prime numbers or calculate them
    checkTime('Start', $timer);
    
    if(file_exists($url)) {
        $prime = unserialize(file_get_contents($url));
    } else {
        $prime = array(2, 3, 5);
    }
    checkTime('PrimeS', $timer, count($prime));
   
    // find prime factors
    $test = $input;
    $result = array();
    
    $check = 1;
    while($input > $check) { 
    
        checkTime('End', $timer);
        checkTime('PrimeE', $timer, count($prime));
        if(checkTime('Limit', $timer, $timeLimit)) {
            $inputFormatted = number_format($input);
            $max = number_format(max($prime));
            $learned = checkTime('PrimeNew', $timer);
            $feedback  = "Hello!<br><br>";
            $feedback .= "I learned {$learned} new prime numbers. I'm now ";
            $feedback .= "up to {$max} but ";
            $feedback .= "I ran out of time to get all the factors of {$inputFormatted}.<br><br>\n";
            $feedback .= "Try again!<br><br>\n";
            $color = 'pink';
            break;
        }
    
        foreach($prime AS $value) {
            while ($test % $value == 0) {
                $check *= $value;
                $result[] = $value;
                $test = $test / $value;
                if($value > $test) { break 2; }
            }
        }
        if($input > $check) {
            // find next prime number
            primeSieve(max($prime)+1, $prime, $url);
        }

    } // while($input > $check)
    

    // output result
    checkTime('End', $timer);
    checkTime('Time', $timer);
    $time = checkTime('Time', $timer);
        
    if(isset($prime))  {
        $max = number_format(max($prime));
        $count = number_format(count($prime));
    } else {
        $max = 0;
        $count = 0;
    }
    if($feedback == '') {
        $feedback  = "Hello!<br><br>";
        $feedback .= "So far I have learned {$count} prime numbers ";
        $feedback .= "from 2 to {$max}.<br><br>";
        $color = 'green';
    }

    break;

case 'GET':

    if(file_exists($url)) {
        $prime = unserialize(file_get_contents($url));
        $max = number_format(max($prime));
        $count = number_format(count($prime));
        $input = max($prime)+2;
        $feedback  = "Hello!<br><br>";
        $feedback .= "I know {$count} prime numbers from 2 to {$max} ";
        $feedback .= "and I'm eager to learn more.<br><br>";
    } else {
        $feedback  = "Hello!<br><br>";
        $feedback .= "I have not yet learned any prime numbers ";
        $feedback .= "but I'm eager to start learning.<br><br>";
    }
    $color = 'green';

    break;

} // switch ($_SERVER['REQUEST_METHOD'])

?>
<!DOCTYPE html>
<html>
<head>
<meta http-equiv="content-type" content="text/html; charset=iso-8859-1">
<title>Find the Prime Factors of a Number</title>
<style>
.container {
  width: 765px;
  border: 2px solid gray;
}
.help {
  float:right;
  width:250px;
  padding:15px;
  font-size:0.9em;
}
.display {;
  border-right:2px solid gray;
  width: 450px;
  min-height:400px;
  padding:15px;
}
.pink {
  background-color:lightsalmon;
}
.green {
  background-color:palegreen;
}
.white {
  background-color:white;
}
h1 {
  font-size:1.4em;
  margin-top:0px;
}

</style>
</head>
<body>
<div class="container">
<div class="help">
<h1>The Learning Script</h1>
This is an example of a script that learns as it goes along.<br><br>
To calculate the prime factors of a number it's best to have the list of prime numbers because calculating prime numbers is a time consuming algorithm.<br><br>
This script calculates the prime numbers it needs and stores them for reuse, in effect "learning" the prime numbers.<br><br>
<span style="font-size:0.8em;"><strong>Note: </strong>This demo script is limited to <?php echo $timeLimit; ?> seconds execution time so as not to hog the shared server.</span>
</div>
<div class="display <?php echo $color; ?>">
<?php echo $feedback; ?>
<div style="padding:10px;margin-bottom:10px;" class="white" >
<form action="<?php echo $_SERVER['PHP_SELF']; ?>" method="post">
<fieldset>
<legend>Find the Prime Factors of a Number</legend>
  Enter a whole number: <br>
  <input name="number" id="number" type="text" size="10" maxlength="8" value="<?php echo $input; ?>"><br>
  <button type="submit" name="action" value="Factors">Find Prime Factors</button>
  <button type="reset">Reset</button><br><br>
</fieldset>
</form>
</div>
<?php 

if($_SERVER['REQUEST_METHOD'] == 'POST') {
  echo "time= {$time} seconds<br>\n";
  echo "input= {$input}<br>\n";
  echo "<pre>factors\n";
  foreach($result AS $key => $value) {
      echo "{$key} => {$value}\n";
  }
  echo "</pre>\n";
}

?>
</div>
</div>
</body>
</html>
